package cz.zcu.ups.arcaoftpd.client;

import java.net.Socket;
import java.util.Date;

import cz.zcu.ups.arcaoftpd.ArcaoFTPd;
import cz.zcu.ups.arcaoftpd.connection.ActiveConnection;
import cz.zcu.ups.arcaoftpd.connection.Connection;
import cz.zcu.ups.arcaoftpd.users.User;

/**
 * @author Arcao
 *
 */
public class ClientParameters {
	/**
	 *
	 */
	private Socket socket;
	private String path = "/";
	private Connection dataConnection = null;
	private boolean logged = false;
	private boolean closed = false;
	private long fileSeekPosition = 0;
	private TransferMode transferMode;
	private String renameFromFilename = "";
	private User user = null;
	private String username = "";
	private String encoding = "cp1250";


	/**
	 * Konstruktor tridy
	 * @param socket	socket, na ktery je pripojen klient
	 */
	public ClientParameters(Socket socket) {
		this.socket = socket;
	}
	/**
	 * Nastavi socket klienta
	 * @param socket	socket, na ktery je pripojen klient
	 * @return			referenci na (this) ClientParameters
	 */
	public ClientParameters setSocket(Socket socket) {
		this.socket = socket;
		return this;
	}
	/**
	 * Vraci socket klienta
	 * @return 			socket klienta
	 */
	public Socket getSocket() {
		return this.socket;
	}

	/**
	 * Nastavuje aktualni adresar klienta
	 * @param path		aktualni adresar klienta
	 * @return			referenci na (this) ClientParameters
	 */
	public ClientParameters setPath(String path) {
		this.path = path;
		return this;
	}
	/**
	 * Vraci aktualni adresar klienta
	 * @return			aktualni adresar klienta
	 */
	public String getPath() {
		return this.path;
	}

	/**
	 * Nastavuje uzivatelske jmeno aktualne prihlaseneho klienta
	 * @param username	uzivatelske jmeno klienta
	 * @return			referenci na (this) ClientParameters
	 */
	public ClientParameters setUsername(String username) {
		this.username = username;
		user = null;
		logged = false;
		return this;
	}

	/**
	 * Vraci uzivatelske jmeno aktualne prihlaseneho klienta
	 * @return			uzivatelske jmeno klienta
	 */
	public String getUsername() {
		if (user == null) {
			return username;
		} else {
			return user.getUsername();
		}
	}

	/**
	 * Nastavuje datove spojeni
	 * @param dataConnection	datove spojeni
	 * @return			objekt ClientParameters
	 */
	public ClientParameters setDataConnection(Connection dataConnection) {
		this.dataConnection = dataConnection;
		return this;
	}
	/**
	 * Vraci datove spojeni
	 * @return			datove spojeni
	 */
	public Connection getDataConnection() {
		return dataConnection;
	}
	/**
	 * Vraci rezim prenosu aktivni/pasivni z pohledu serveru
	 * @return			pouziva se aktivni rezim prenosu?
	 */
	public boolean getActiveMode() {
		if (dataConnection == null) return false;
		return (dataConnection instanceof ActiveConnection);
	}
	/**
	 * Nastavuje, zda je uzivatel prihlasen
	 * @param logged	uzivatel prihlasen?
	 * @return			objekt ClientParameters
	 * @deprecated		nastavuje se nyni podle toho, jestli parametr do setUser je null ci ne
	 */
	public ClientParameters setLogged(boolean logged) {
		this.logged = logged;
		return this;
	}
	/**
	 * Vraci, zda je uzivatel prihlasen
	 * @return			uzivatel prihlasen?
	 */
	public boolean getLogged() {
		return this.logged;
	}
	/**
	 * Nastavuje, zda je spojeni ukonceno
	 * @param closed	spojeni ukonceno?
	 * @return			objekt ClientParameters
	 */
	public ClientParameters setClosed(boolean closed) {
		this.closed = closed;
		return this;
	}
	/**
	 * Vraci, zda je spojeni ukonceno
	 * @return			spojeni ukonceno?
	 */
	public boolean getClosed() {
		return this.closed;
	}
	/**
	 * Nastavuje pozici, od ktere se ma soubor stahovat
	 * @param fileSeekPosition	pozice, od ktere se ma soubor stahovat
	 * @return			objekt ClientParameters
	 */
	public ClientParameters setFileSeekPosition(long fileSeekPosition) {
		this.fileSeekPosition = fileSeekPosition;
		return this;
	}
	/**
	 * Vraci pozici, od ktere se ma soubor stahovat
	 * @return			pozice, od ktere se ma soubor stahovat
	 */
	public long getFileSeekPosition() {
		return this.fileSeekPosition;
	}

	/**
	 * Vraci rezim prenosu (ascii, binary)
	 * @return			rezim prenosu
	 */
	public TransferMode getTransferMode() {
		return transferMode;
	}

	/**
	 * Nastavuje rezim prenosu (ascii, binary)
	 * @param transferMode	rezim prenosu
	 * @return				reference na (this) ClientParameters
	 */
	public ClientParameters setTransferMode(TransferMode transferMode) {
		this.transferMode = transferMode;
		return this;
	}

	/**
	 * Nastavuje nazev souboru/adresare, ktery se bude prejmenovavat
	 * @param renameFromFilename	nazev souboru/adresare
	 * @return						referenci na (this) ClientParameters
	 */
	public ClientParameters setRenameFromFilename(String renameFromFilename) {
		this.renameFromFilename = renameFromFilename;
		return this;
	}

	/**
	 * Vraci nazev souboru/adresare, ktery se bude prejmenovavat
	 * @return				nazev souboru/adresare
	 */
	public String getRenameFromFilename() {
		return renameFromFilename;
	}

	/**
	 * Nastavuje informace o uzivateli
	 * @param user			informace o uzivateli
	 * @return				referenci na (this) ClientParameters
	 */
	public ClientParameters setUser(User user) {
		this.user = user;
		logged = (user != null);
		return this;
	}

	/**
	 * Vraci informace o uzivateli
	 * @return			informace o uzivateli
	 */
	public User getUser() {
		return user;
	}

	/**
	 * Vraci kodovani prikazu a parametru (UTF-8 / windows-1250)
	 * @return			kodovani prikazu
	 */
	public String getEncoding() {
		return encoding;
	}

	/**
	 * Nastavuje kodovani prikazu a parametru (UTF-8 / windows-1250)
	 * @param encoding	kodovani prikazu
	 * @return			reference na (this) ClientParameters
	 */
	public ClientParameters setEncoding(String encoding) {
		this.encoding = encoding;
		return this;
	}

	/**
	 * Odesle formatovanou odpoved do socketu
	 * @param responseCode		kod opdovedi
	 * @param message			text odpovedi, moznost formatovani skrz formatovaci parametry u tridy Formatter
	 * @param args				parametry pro moznost formatovani
	 * @see java.util.Formatter
	 */
	public void sendMessage(int responseCode, String message, Object... args) {
		//pokud je odpoved viceradkova, provedeme upravu, aby to byla spravna odpoved pro FTP RFC
		if (message.indexOf('\n') > 0) {
			String[] lines = message.split("\\s*(\\n+\\s*)+");
			StringBuffer messageSB = new StringBuffer();

			//projedem vsechny radky
			for (int i = 0; i < lines.length; i++) {
				String item = lines[i].trim();

				//prvni radek vypada: kod-odpoved
				if (i == 0) {
					messageSB.append(responseCode);
					messageSB.append("-");
				//dalsi radky jsou odsazeny mezerou
				} else if (i != lines.length - 1) {
					messageSB.append(" ");
				//posledni radek vypad: kod odpoved
				} else {
					messageSB.append(responseCode);
					messageSB.append(" ");
				}
				messageSB.append(item).append('\n');
			}
			message = messageSB.toString();
		//pokud je odpoved jednoradkova
		} else {
			message = new StringBuffer()
					  .append(responseCode)
					  .append(" ")
					  .append(message)
					  .append('\n')
					  .toString();
		}

		//provedeme formatovani
		String res;
		if (args.length > 0) {
			res = String.format(message, args);
		} else {
			res = message;
		}

		try {
			//logovat, pokud je to mozne
			if (ArcaoFTPd.getLogger() != null)
				ArcaoFTPd.getLogger().write(("S: " + new Date().toString() + "; " + socket.getRemoteSocketAddress().toString() +": "+ res).getBytes("utf-8"));

			//zasleme odpoved
			socket.getOutputStream().write(res.getBytes(encoding));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}